package java.nio.channels;

import java.nio.ByteBuffer;
import java.net.Socket;
import java.net.SocketOption;
import java.net.SocketAddress;
import java.io.IOException;
import java.nio.channels.spi.SelectorProvider;
import java.nio.channels.spi.AbstractSelectableChannel;

public abstract class SocketChannel extends AbstractSelectableChannel implements ByteChannel, ScatteringByteChannel, GatheringByteChannel, NetworkChannel
{
    protected SocketChannel(SelectorProvider provider) {
        super(provider);
    }
    
    public static SocketChannel open() throws IOException {
        return SelectorProvider.provider().openSocketChannel();
    }
    
    public static SocketChannel open(SocketAddress socketAddress) throws IOException {
        SocketChannel open = open();
        try {
            open.connect(socketAddress);
        }
        catch (Throwable t) {
            try {
                open.close();
            }
            catch (Throwable exception) {
                t.addSuppressed(exception);
            }
            throw t;
        }
        assert !(!open.isConnected());
        return open;
    }
    
    @Override
    public final int validOps() {
        return 13;
    }
    
    @Override
    public abstract SocketChannel bind(SocketAddress p0) throws IOException;
    
    @Override
    public abstract <T> SocketChannel setOption(SocketOption<T> p0, T p1) throws IOException;
    
    public abstract SocketChannel shutdownInput() throws IOException;
    
    public abstract SocketChannel shutdownOutput() throws IOException;
    
    public abstract Socket socket();
    
    public abstract boolean isConnected();
    
    public abstract boolean isConnectionPending();
    
    public abstract boolean connect(SocketAddress p0) throws IOException;
    
    public abstract boolean finishConnect() throws IOException;
    
    public abstract SocketAddress getRemoteAddress() throws IOException;
    
    @Override
    public abstract int read(ByteBuffer p0) throws IOException;
    
    @Override
    public abstract long read(ByteBuffer[] p0, int p1, int p2) throws IOException;
    
    @Override
    public final long read(ByteBuffer[] array) throws IOException {
        return this.read(array, 0, array.length);
    }
    
    @Override
    public abstract int write(ByteBuffer p0) throws IOException;
    
    @Override
    public abstract long write(ByteBuffer[] p0, int p1, int p2) throws IOException;
    
    @Override
    public final long write(ByteBuffer[] array) throws IOException {
        return this.write(array, 0, array.length);
    }
    
    @Override
    public abstract SocketAddress getLocalAddress() throws IOException;
}
